@@ -1,15 +1,17 @@
 import asyncio
-from modulefinder import Module
 import platform
 
 import nest_asyncio
 
 from .utils.aid_bvid_transformer import aid2bvid, bvid2aid
 from .utils.Credential import Credential
-from .utils.Danmaku import Danmaku
-from .utils.network import get_session, request, set_session
+from .utils.Danmaku import Danmaku, DmFontSize, DmMode
 from .utils.short import get_real_url
 from .utils.sync import sync
+from .utils.network_httpx import get_session, set_session
+from .utils.parse_link import parse_link
+
+HEADERS = {"User-Agent": "Mozilla/5.0", "Referer": "https://bilibili.com"}
 
 # 支持嵌套事件
 nest_asyncio.apply()
